#pragma include "utils/Preferences.vert.glsl"
#pragma include "DefaultCommon.glsl"

mat4 getObjectToScreenMatrix(); // Returns a matrix that converts a point from an object's coordinates to screen coordinates.
void setVaryingPositions();     // Sets the position, screen position and anchor position to the corresponding variyngs
vec2 getUv();                   // Returns the uv coordinate from a_uv. If the object is not textured, returns 0, 0
void setVaryingUvs();           // Sets the uv coordinates to the appropriate varying
lowp vec4 getWholeColor();      // Returns the color from the uniform. If there is no color or the element is drawn in a batch, it returns white color.
lowp vec4 getColor();           // Returns the resulting color from the uniform and attribute.
void setVaryingColors();        // Sets the colors to varying
void defaultMain();             // Run the main logic of the uber shader, fill in the values ​​in varyings

// textures and tiles
#if defined(Z_TEXTURED)
    attribute vec2 a_uv;
    varying vec2 v_uv;
#endif

// colors
#if defined(Z_VERTEX_COLORED)
    attribute vec4 a_color;
    varying vec4 v_color; // Color from vertex attribute

    #if !defined(Z_BATCHED)
        varying vec4 v_precalculatedColor; // Result color from uniforms and attributes
    #endif
#endif

#if !defined(Z_BATCHED)
    uniform lowp vec4 u_color;
    uniform mat4 u_model;
#endif

attribute vec2 a_position;
uniform mat4 u_view;
uniform mat4 u_projection;

varying vec2 v_position; // Point position in screen coordinates
varying vec2 v_screenPosition;

uniform mat4 u_anchorTransform;
varying vec2 v_anchorPosition; // The position of the point in the coordinates of the anchor

mat4 getObjectToScreenMatrix() {
    mat4 matrix = u_projection * u_view;
    #if !defined(Z_BATCHED)
        matrix *= u_model;
    #endif

    return matrix;
}

void setVaryingPositions() {
    gl_Position = getObjectToScreenMatrix() * vec4(a_position, 0.0, 1.0);
    v_position = a_position;
    v_screenPosition = gl_Position.xy;
    v_anchorPosition = (u_anchorTransform * vec4(v_screenPosition, 0.0, 1.0)).xy;
}

vec2 getUv() {
    #if defined(Z_TEXTURED)
        return a_uv;
    #else
        return vec2(0, 0);
    #endif
}

void setVaryingUvs() {
    #if defined(Z_TEXTURED)
        v_uv = getUv();
    #endif
}

lowp vec4 getWholeColor() {
    #if !defined(Z_BATCHED)
        return u_color;
    #else
        return vec4(1, 1, 1, 1);
    #endif
}

lowp vec4 getVertexColor() {
    #if defined(Z_VERTEX_COLORED)
        return a_color;
    #else
        return vec4(1, 1, 1, 1);
    #endif
}

lowp vec4 getColor() {
    return getWholeColor() * getVertexColor();
}

void setVaryingColors() {
    #if defined(Z_VERTEX_COLORED)
        v_color = getVertexColor();

        #if !defined(Z_BATCHED)
            v_precalculatedColor = getWholeColor() * v_color;
        #endif
    #endif
}

void defaultMain() {
    setVaryingPositions();
    setVaryingColors();
    setVaryingUvs();
}
